PRAGMA foreign_keys = ON;;

CREATE TRIGGER maj_injections_update AFTER UPDATE OF GEOMETRY ON injections
BEGIN
    INSERT INTO noeuds_injections(noeud, injection)
    SELECT n.OGC_FID, i.OGC_FID FROM noeuds as n, injections as i
    WHERE Intersects(n.GEOMETRY, i.GEOMETRY)
    AND i.OGC_FID = NEW.OGC_FID
    UNION
    SELECT r.noeud, r.injection FROM (
      SELECT i.OGC_FID AS injection, n.OGC_FID AS noeud, MIN(Distance(n.GEOMETRY, i.GEOMETRY)) as d
      FROM injections AS i, noeuds AS n, mailles AS m
      WHERE Intersects(m.GEOMETRY, n.GEOMETRY)
      AND n.ROWID IN (SELECT ROWID FROM SpatialIndex
                      WHERE f_table_name = 'noeuds'
                      AND search_frame = m.GEOMETRY)
      AND Intersects(i.GEOMETRY, m.GEOMETRY)
      AND m.ROWID IN (SELECT ROWID FROM SpatialIndex
                      WHERE f_table_name = 'mailles'
                      AND search_frame = i.GEOMETRY)
      AND i.OGC_FID = NEW.OGC_FID) AS r WHERE d > 0
    ;

    UPDATE injections SET
      permeabilite = (
        SELECT Coalesce(EXP(SUM(LN(n.permeabilite_zns))/COUNT(1)),
                         EXP(SUM(LN(n.permeabilite_x))/COUNT(1)))
        FROM noeuds AS n, noeuds_injections AS ni
        WHERE ni.injection = injections.OGC_FID
        AND n.OGC_FID = ni.noeud ),
      infiltration = (
        SELECT AVG(n.infiltration)
        FROM noeuds AS n, noeuds_injections AS ni
        WHERE ni.injection = injections.OGC_FID
        AND n.OGC_FID = ni.noeud ),
      epaisseur = (
        SELECT MIN(n.epaisseur_zns)
        FROM noeuds AS n, noeuds_injections AS ni
        WHERE ni.injection = injections.OGC_FID
        AND n.OGC_FID = ni.noeud ),
      altitude = (
        SELECT AVG(n.altitude)
        FROM noeuds AS n, noeuds_injections AS ni
        WHERE ni.injection = injections.OGC_FID
        AND n.OGC_FID = ni.noeud )
      WHERE injections.OGC_FID = NEW.OGC_FID
    ;
END
;;

CREATE TRIGGER maj_injections_insert AFTER INSERT ON injections
BEGIN
    INSERT INTO noeuds_injections(noeud, injection)
    SELECT n.OGC_FID, i.OGC_FID FROM noeuds as n, injections as i
    WHERE Intersects(n.GEOMETRY, i.GEOMETRY)
    AND i.OGC_FID = NEW.OGC_FID
    UNION
    SELECT r.noeud, r.injection FROM (
      SELECT i.OGC_FID AS injection, n.OGC_FID AS noeud, MIN(Distance(n.GEOMETRY, i.GEOMETRY)) as d
      FROM injections AS i, noeuds AS n, mailles AS m
      WHERE Intersects(m.GEOMETRY, n.GEOMETRY)
      AND n.ROWID IN (SELECT ROWID FROM SpatialIndex
                      WHERE f_table_name = 'noeuds'
                      AND search_frame = m.GEOMETRY)
      AND Intersects(i.GEOMETRY, m.GEOMETRY)
      AND m.ROWID IN (SELECT ROWID FROM SpatialIndex
                      WHERE f_table_name = 'mailles'
                      AND search_frame = i.GEOMETRY)
      AND i.OGC_FID = NEW.OGC_FID) AS r WHERE d > 0;

    UPDATE injections SET
      permeabilite = (
        SELECT Coalesce(EXP(SUM(LN(n.permeabilite_zns))/COUNT(1)),
                         EXP(SUM(LN(n.permeabilite_x))/COUNT(1)))
        FROM noeuds AS n, noeuds_injections AS ni
        WHERE ni.injection = injections.OGC_FID
        AND n.OGC_FID = ni.noeud ),
      infiltration = (
        SELECT AVG(n.infiltration)
        FROM noeuds AS n, noeuds_injections AS ni
        WHERE ni.injection = injections.OGC_FID
        AND n.OGC_FID = ni.noeud ),
      epaisseur = (
        SELECT MIN(n.epaisseur_zns)
        FROM noeuds AS n, noeuds_injections AS ni
        WHERE ni.injection = injections.OGC_FID
        AND n.OGC_FID = ni.noeud ),
      altitude = (
        SELECT AVG(n.altitude)
        FROM noeuds AS n, noeuds_injections AS ni
        WHERE ni.injection = injections.OGC_FID
        AND n.OGC_FID = ni.noeud )
      WHERE injections.OGC_FID = NEW.OGC_FID
    ;
END
;;
