# -*- coding: utf-8 -*-
"""
Import tiles

USAGE
    python -m thyrsis.import_tiles [-h, -z<level>, -u<url>, -n<name>] -c<xmin,xmax,ymin,ymax> output_dir
"""
from builtins import map
from builtins import str
from builtins import range

import subprocess
import math
import os
import sys
import urllib.request, urllib.error, urllib.parse
import getopt

from ..log import logger

if __name__=="__main__":
    try:
        optlist, args = getopt.getopt(sys.argv[1:],
                "hz:u:n:c:, ")
    except Exception as e:
        sys.stderr.write(str(e)+"\n")
        exit(1)

    optlist = dict(optlist)

    if "-h" in optlist or "--help" in optlist:
        help(sys.modules[__name__])
        exit(0)

    ech = 6378137.
    coef = ech*math.pi

    assert len(sys.argv) >= 2

    #parametres#############################################################
    # URL tiles to import
    http_tiles = optlist['-u'] if '-u'in optlist else 'https://tile.thunderforest.com/landscape/'
    # répertoire destination
    tiles_dir = args[0]
    if not os.path.isdir(tiles_dir):
        raise RuntimeError("tiles directory do not exist : "+tiles_dir)
    # niveau de zoom
    z = int(optlist['-z']) if '-z' in optlist else 12
    # nom de la carte à construire
    nom = optlist['-n'] if '-n' in optlist else "tiles"
    # coordonnées du rectangle à importer
    coord = list(map(int,optlist['-c'].split(','))) if '-c' in optlist else []
    logger.notice(coord)
    if len(coord) == 0:
        raise RuntimeError("les coordonnées diovent être indiquées avec l'option -c")
    xmin = coord[0]
    xmax = coord[1]
    ymin = coord[2]
    ymax = coord[3]
    ########################################################################
    logger.notice("xmin, xmax, ymin, ymax = ", xmin, xmax, ymin, ymax)

    # pour info, transfo en longitude, latitude (pas utile)
    lonw = 180.*xmin/coef
    lone = 180.*xmax/coef
    latn = math.atan(math.exp(math.pi*ymax/coef))*360/math.pi-90
    lats = math.atan(math.exp(math.pi*ymin/coef))*360/math.pi-90
    logger.notice("lonw, latn, lone, lats = ", lonw, latn, lone, lats)

    zoom_dir = os.path.join(tiles_dir,str(z))
    if not os.path.exists(zoom_dir):
        logger.notice("creating directory ", zoom_dir)
        os.mkdir(zoom_dir)

    n = 2**z

    imin = int(0.5*n*(xmin/coef+1))
    imax = int(0.5*n*(xmax/coef+1))
    jmax = int(0.5*n*(1-ymin/coef))
    jmin = int(0.5*n*(1-ymax/coef)) - 1
    logger.notice(imin, imax, jmin, jmax)
    logger.notice((2.*imin/n-1)*coef, (2.*(imax+1)/n-1)*coef, (1-2.*(jmax+1)/n)*coef, (1-2.*jmin/n)*coef)

    tiles_tif = ""
    for i in range(imin, imax+1):
        x_dir = os.path.join(zoom_dir,str(i))
        if not os.path.exists(x_dir):
            logger.notice("creating directory ", x_dir)
            os.mkdir(x_dir)
        for j in range(jmin, jmax+1):
            xw = (2.*i/n-1)*coef
            xe = (2.*(i+1)/n-1)*coef
            yn = (1-2.*j/n)*coef
            ys = (1-2.*(j+1)/n)*coef
    #        logger.notice(i, j, xw, yn, xe, ys)
            png_file = os.path.join(x_dir, str(j)+'.png')
            tiff_file = os.path.join(x_dir, str(j)+'.tiff')
            if os.path.exists(png_file):
                logger.notice(png_file, " exists")
            else:
                tile_url = http_tiles+str(z)+'/'+str(i)+'/'+str(j)+'.png'
                logger.notice("request ", tile_url)
                response = urllib.request.urlopen(tile_url)
                open(png_file, 'w').write(response.read())
            tiles_tif += ' '+tiff_file
            if os.path.exists(tiff_file):
                logger.notice(tiff_file, " exists")
            else:
                gdal_command = 'gdal_translate -expand rgba -a_nodata 0 -of GTiff -a_srs EPSG:3857 -a_ullr '+str(xw)+' '+str(yn)+' '+str(xe)+' '+str(ys)+' '+png_file+' '+tiff_file
                logger.notice(gdal_command)
                subprocess.call(gdal_command, shell=True)

    vrt_command = 'gdalbuildvrt -a_srs EPSG:3857 %s %s'%(os.path.join(tiles_dir, '%s.vrt'%(nom)), tiles_tif)
    logger.notice(vrt_command)
    subprocess.call(vrt_command, shell=True)
